//*****************************************************************************
// uart.c - UART communication
//
// Company Name- Zera India Private limited.  
// Author - Rajesh Varma 
// Date - 04/09/2017
// Revision - 0
//
// Remarks
// UART Setting 9600 bits per second, and 8-n-1 mode
//
//*****************************************************************************

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "inc/hw_memmap.h"
#include "inc/hw_ints.h"
#include "inc/hw_types.h"
#include "driverlib/sysctl.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/sysctl.h"
#include "driverlib/timer.h"
#include "driverlib/rom.h"
#include "driverlib/rom_map.h"
#include "driverlib/uart.h"
#include "utils/uartstdio.h"
#include "grlib/grlib.h"
#include "drivers/frame.h"
#include "drivers/kentec320x240x16_ssd2119.h"
#include "drivers/pinout.h"
#include "macros.h"
#include "extern_variables.h"

#define Offset 5        // offset for accesing the falsh address for the constants
//*****************************************************************************
//
// The variable defination used in uart communication
//
//*****************************************************************************

volatile unsigned char *write_ptr_uart2;
unsigned char *read_ptr_uart2;
unsigned char *rcv_buff_end_uart2;


unsigned char pc_buff[20];
unsigned char ok_length;
unsigned char *write_ptr;
unsigned char *read_ptr;
unsigned char *rcv_buff_end;


volatile unsigned char *write_ptr_uart0;
unsigned char *read_ptr_uart0;
unsigned char *rcv_buff_end_uart0;
unsigned char send_flag;
unsigned char trans_mode;
unsigned char retry_cnt;
unsigned char pre_data[3];
unsigned char sample;


unsigned int count;
unsigned int length_cnt;
unsigned int trans_length3;
unsigned int trans_data_cnt;
unsigned int test_count_2,dummyyy;


volatile unsigned char complete_data_flag;


unsigned char data[1];
unsigned char QD_Flag;
void response_meter(void);
void tmipdisplay(void);
int Hex_To_Dec(unsigned char[]);



//**************************************************************************************************
///
/// UART0 Intialisation
/// I/p: Nothing
/// O/P: noting
/// Remark: UART0 Intialisation for communication with Salves on RS485 Bus 
//***************************************************************************************************/
void Uart0_init(uint32_t ui32SysClock)
{

    MAP_GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);    // Initialize the UART as a console for text I/O.

    MAP_UARTConfigSetExpClk(UART0_BASE, ui32SysClock, 9600,
                            (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                             UART_CONFIG_PAR_NONE));

    MAP_IntEnable(INT_UART0);
    MAP_UARTIntEnable(UART0_BASE, UART_INT_RX | UART_INT_RT);

    write_ptr_uart0 = cir_buffer_uart0;
    read_ptr_uart0 = cir_buffer_uart0;
    rcv_buff_end_uart0 = &cir_buffer_uart0[SIZE];



 }

//**************************************************************************************************
///
/// UART0 Interrupt Handler
/// I/p: Nothing
/// O/P: noting
/// Remark: UART0 Interrupt handler for communication between salves on RS485-Bus 
//***************************************************************************************************/
//unsigned char temp0,var123;
void UARTIntHandler_0(void)
{
    unsigned long ulStatus;

        ulStatus = MAP_UARTIntStatus(UART0_BASE, true);  // Get the interrrupt status.
        MAP_UARTIntClear(UART0_BASE, ulStatus);         // Clear the asserted interrupts.


        while(MAP_UARTCharsAvail(UART0_BASE))   // Loop while there are characters in the receive FIFO.
        {
            // Read the next character from the UART
            *write_ptr_uart0 = MAP_UARTCharGetNonBlocking(UART0_BASE);
            if(transperent_mode==0xFF)
            {
                data[0]= *write_ptr_uart0;
                UARTSend_PC(&data[0] ,1);
                UAR0_Received_flag=1;
                pre_data[0]=data[0];
                for(sample=2;sample>0;sample--)
                {
                    pre_data[sample]=pre_data[sample-1];
                    if(pre_data[2]=='D' && pre_data[1]=='O')
                    {
                     c_pre_data[2]=pre_data[2];
                     c_pre_data[1]=pre_data[1];
                     DISABLE_PIN();
                     transperent_mode=0;

                    }
                    if(pre_data[2]=='Q' && pre_data[1]=='D')
                    {
                     c_pre_data[2]=pre_data[2];
                     c_pre_data[1]=pre_data[1];
                     DISABLE_PIN();
                     transperent_mode=0;

                    }
                }
            }
            else
            {
                if( *write_ptr_uart0=='\r')
                {
                     UAR0_Received_flag=1;
                }
            }

            ++write_ptr_uart0;
            trans_data_cnt++;

            if(write_ptr_uart0 == rcv_buff_end_uart0)
                write_ptr_uart0 = cir_buffer_uart0;
        }



}

//*****************************************************************************
/// void UART\(const unsigned char *pucBuffer, unsigned long ulCount)
/// I/P: string buffer and no. of count
/// O/P: Send a string to the UART.
///
//*****************************************************************************

void UARTSend(const unsigned char *pucBuffer, unsigned long ulCount)
{
    // Loop while there are more characters to send.
    while(ulCount--)
    {
        MAP_UARTCharPutNonBlocking(UART0_BASE, *pucBuffer++);  // Write the next character to the UART.
    }
}
 
//*****************************************************************************************
///
/// UART1_PC_INIT function
/// I/p: Nothing
/// O/P: noting
/// Remark: UART1 intialisation for communication with PC.
//*****************************************************************************************/
void Uart1_pc_init(uint32_t ui32SysClock)
{
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_UART1); //    Enable the (non-GPIO) peripherals used by this example.
    MAP_GPIOPinTypeUART(GPIO_PORTB_BASE, GPIO_PIN_4 | GPIO_PIN_5);   // Set GPIO B4 and B5 as UART pins.

    MAP_UARTConfigSetExpClk(UART1_BASE, ui32SysClock, 9600,
                            (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                             UART_CONFIG_PAR_NONE));

  //    UARTEchoSet(false);

   MAP_IntEnable(INT_UART1);
   MAP_UARTIntEnable(UART1_BASE, UART_INT_RX | UART_INT_RT);


    write_ptr = cir_buffer;
    read_ptr = cir_buffer;
    rcv_buff_end = &cir_buffer[SIZE];

}


//*****************************************************************************
///
/// The UART1 interrupt handler.
///
//*****************************************************************************
void UARTIntHandler_1(void)
{
    unsigned long ulStatus;
        
        ulStatus = MAP_UARTIntStatus(UART1_BASE, true);  // Get the interrrupt status.
        MAP_UARTIntClear(UART1_BASE, ulStatus);      // Clear the asserted interrupts.

        while(MAP_UARTCharsAvail(UART1_BASE))        // Loop while there are characters in the receive FIFO.
        {

            *write_ptr = MAP_UARTCharGetNonBlocking(UART1_BASE);  // Read the next character from the UART

            if(transperent_mode==0xFF)
            {
                ENABLE_PIN();
                data[0]= *write_ptr;
                UARTSend(&data[0] ,1);
                UAR1_Received_flag=1;
        //      DISABLE_PIN();
            }

            write_ptr++;
            Length_Received_Buff++;


            if(write_ptr == rcv_buff_end)write_ptr = cir_buffer;
        UAR1_Received_flag=1;
        }


}


//*****************************************************************************
/// void UARTSend(const unsigned char *pucBuffer, unsigned long ulCount)
/// I/P: string buffer and no. of count
/// O/P: Send a string to the UART.
///
//*****************************************************************************

void UARTSend_PC(const unsigned char *pucBuffer, unsigned long ulCount)
{
    // Loop while there are more characters to send.
    while(ulCount--)
    {
        MAP_UARTCharPutNonBlocking(UART1_BASE, *pucBuffer++);  // Write the next character to the UART.
    }
}

//*****************************************************************************************
///
/// UART2_Bluetooth_INIT function
/// I/p: Nothing
/// O/P: noting
/// Remark: UART12 intialisation for Bluetooth communication.
//*****************************************************************************************/
void Uart2_Bluetooth_init(uint32_t ui32SysClock)
{
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_UART2); //    Enable the (non-GPIO) peripherals used by this example.
    MAP_GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_6 | GPIO_PIN_7);   // Set GPIO A6 and  as UART pins.

    MAP_UARTConfigSetExpClk(UART2_BASE, ui32SysClock, 9600,
                            (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                             UART_CONFIG_PAR_NONE));


   MAP_IntEnable(INT_UART2);
   MAP_UARTIntEnable(UART2_BASE, UART_INT_RX | UART_INT_RT);


    write_ptr_uart2 = cir_buffer_2;
    read_ptr_uart2 = cir_buffer_2;
    rcv_buff_end_uart2 = &cir_buffer_2[SIZE];

}


//*****************************************************************************
///
/// The UART1 interrupt handler.
///
//*****************************************************************************
void UARTIntHandler_2(void)
{
    unsigned long ulStatus;

        ulStatus = MAP_UARTIntStatus(UART2_BASE, true);  // Get the interrrupt status.
        MAP_UARTIntClear(UART2_BASE, ulStatus);      // Clear the asserted interrupts.

        while(MAP_UARTCharsAvail(UART2_BASE))        // Loop while there are characters in the receive FIFO.
        {

            *write_ptr_uart2 = MAP_UARTCharGetNonBlocking(UART2_BASE);  // Read the next character from the UART

            if(transperent_mode==0xFF)
            {
                ENABLE_PIN();
                data[0]= *write_ptr_uart2;
                UARTSend_BL(&data[0] ,1);
                UAR2_Received_flag=1;
        //      DISABLE_PIN();
            }

            write_ptr_uart2++;
            Length_Received_Buff++;


            if(write_ptr_uart2 == rcv_buff_end_uart2)write_ptr_uart2 = cir_buffer_2;
        UAR2_Received_flag=1;
        }


}


//*****************************************************************************
/// void UARTSend(const unsigned char *pucBuffer, unsigned long ulCount)
/// I/P: string buffer and no. of count
/// O/P: Send a string to the UART.
///
//*****************************************************************************

void UARTSend_BL(const unsigned char *pucBuffer, unsigned long ulCount)
{
    // Loop while there are more characters to send.
    while(ulCount--)
    {
        MAP_UARTCharPutNonBlocking(UART2_BASE, *pucBuffer++);  // Write the next character to the UART.
    }
}


//**********************************************************************************
/// void RS485_Communication
/// I/P: Nothing
/// O/P: Npthing

//**********************************************************************************
unsigned int recieve_count,transmit_count;
long test1;
void RS485_communication(void)
{

        switch(cir_buffer_uart0[1])
        {
            case'A':
                    single_flag=1;
                    continuous_flag=0;
                    if(SC_PULSE_Selection==0)
                        Error_Display_Seq=0;
                    single_continuous();
                    if(disp_para_cnt==4)
                        disp_para_cnt=1;

            break;

            case'B':
                    single_flag=0;             ;
                    continuous_flag=1;
                    single_continuous();
                    if(disp_para_cnt==4)
                        disp_para_cnt=1;

            break;

            case'C':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;
                    Start_postion=atoi(Start_temp);                 // Start position

                    End_temp[0] = cir_buffer_uart0[4] ;
                    End_temp[1] = cir_buffer_uart0[5] ;
                    End_postion=atoi(End_temp);                     //End position


                    if(Meter_Place.Slave_ID_flash== End_postion)
                    {
                        ENABLE_PIN();
                        send_acknowledgement();
                        DISABLE_PIN();
                    }

                    memset(Start_temp,'\0',sizeof(Start_temp));
                    memset(End_temp,'\0',sizeof(End_temp));
                    Clear_UART0_Buffer();

            break;

            case'D':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;

                    Received_ID= atoi(Start_temp);
                    if(cir_buffer_uart0[0]=='Q')
                    {
                        if(Meter_Place.Slave_ID_flash==Received_ID)
                        {
                            transperent_mode=0xFF;
                            ENABLE_PIN();
                            send_acknowledgement();
                        //  DISABLE_PIN();
                            tmipdisplay();
                            QD_Flag=1;
                        }
                        else
                        {
                            transperent_mode=0;
                            DISABLE_PIN();
                            clean_flag=1;
                            disp_para_cnt=1;
                        //  Error_Display_Seq=1;
                            edge_accuracy_switch=0 ;
                        }
                    }
                    else
                    {

                       End_temp[0] = cir_buffer_uart0[4] ;
                       End_temp[1] = cir_buffer_uart0[5] ;
                       End_postion=atoi(End_temp);                      //End position

                       if(Meter_Place.Slave_ID_flash== End_postion)
                        {
                            ENABLE_PIN();
                            send_acknowledgement();
                            DISABLE_PIN();
                        }
                    }

                    memset(Start_temp,'\0',sizeof(Start_temp));
                    memset(End_temp,'\0',sizeof(End_temp));
                    Clear_UART0_Buffer();


            break;

            case'E':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;
                    Start_postion=atoi(Start_temp);                 // Start position

                    End_temp[0] = cir_buffer_uart0[4] ;
                    End_temp[1] = cir_buffer_uart0[5] ;
                    End_postion=atoi(End_temp);                     //End position


                    if(Meter_Place.Slave_ID_flash== End_postion)
                    {
                        ENABLE_PIN();
                        send_acknowledgement();
                        DISABLE_PIN();
                    }

                    memset(Start_temp,'\0',sizeof(Start_temp));
                    memset(End_temp,'\0',sizeof(End_temp));
                    Clear_UART0_Buffer();

            break;

            case'F':

                    Edge_count_fun();
                    first_edges_flag=1;

            break;

            case'G':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;

                    Received_ID= atoi(Start_temp);
                    if(Meter_Place.Slave_ID_flash==Received_ID)
                    {
                        ENABLE_PIN();
                        if(Edge_mode_flag||reset_count_flag)
                            send_edge_results();
                        else
                            send_accuracy_results();

                        DISABLE_PIN();
                    }

                    memset(Start_temp,'\0',sizeof(Start_temp));
                    Clear_UART0_Buffer();

            break;

            case'H':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;

                    Received_ID= atoi(Start_temp);
                    if(Meter_Place.Slave_ID_flash==Received_ID)
                    {
                        ENABLE_PIN();
                        send_version_HWACK();
                        DISABLE_PIN();
                    }

                    memset(Start_temp,'\0',sizeof(Start_temp));
                    Clear_UART0_Buffer();

            break;

            case'I':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;
                    Received_ID= atoi(Start_temp);

                    if(Meter_Place.Slave_ID_flash==Received_ID)
                    {
                        ENABLE_PIN();
                        send_version_SWACK();
                        DISABLE_PIN();
                    }

                    memset(Start_temp,'\0',sizeof(Start_temp));
                    Clear_UART0_Buffer();

            break;

            case'J':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;
                    Start_postion=atoi(Start_temp);                 // Start position

                    End_temp[0] = cir_buffer_uart0[4] ;
                    End_temp[1] = cir_buffer_uart0[5] ;
                    End_postion=atoi(End_temp);                     //End position


                    if(Meter_Place.Slave_ID_flash== End_postion)
                    {
                        ENABLE_PIN();
                        send_acknowledgement();
                        DISABLE_PIN();
                    }

                    memset(Start_temp,'\0',sizeof(Start_temp));
                    memset(End_temp,'\0',sizeof(End_temp));
                    Clear_UART0_Buffer();

            break;

            case'K':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;
                    Start_postion=atoi(Start_temp);                 // Start position

                    End_temp[0] = cir_buffer_uart0[4] ;
                    End_temp[1] = cir_buffer_uart0[5] ;
                    End_postion=atoi(End_temp);                     //End position


                    if(Meter_Place.Slave_ID_flash== End_postion)
                    {
                        ENABLE_PIN();
                        send_acknowledgement();
                        DISABLE_PIN();
                    }

                    memset(Start_temp,'\0',sizeof(Start_temp));
                    memset(End_temp,'\0',sizeof(End_temp));
                    Clear_UART0_Buffer();

            break;

            case'L':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;
                    Start_postion=atoi(Start_temp);                 // Start position

                    End_temp[0] = cir_buffer_uart0[4] ;
                    End_temp[1] = cir_buffer_uart0[5] ;
                    End_postion=atoi(End_temp);                     //End position


                    if(Meter_Place.Slave_ID_flash== End_postion)
                    {
                        ENABLE_PIN();
                        send_acknowledgement();
                        DISABLE_PIN();
                    }

                    reset_count_flag=1;
                    reset_status_flag=2;
                    memset(Start_temp,'\0',sizeof(Start_temp));
                    memset(End_temp,'\0',sizeof(End_temp));
                    Clear_UART0_Buffer();
                    disp_para_cnt=3;

            break;

            case'M':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;
                    Start_postion=atoi(Start_temp);                 // Start position

                    End_temp[0] = cir_buffer_uart0[4] ;
                    End_temp[1] = cir_buffer_uart0[5] ;
                    End_postion=atoi(End_temp);                     //End position


                    if(Meter_Place.Slave_ID_flash== End_postion)
                    {
                        ENABLE_PIN();
                        send_acknowledgement();
                        DISABLE_PIN();
                    }

                    memset(Start_temp,'\0',sizeof(Start_temp));
                    memset(End_temp,'\0',sizeof(End_temp));
                    Clear_UART0_Buffer();

            break;

            case'N':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;
                    Start_postion=atoi(Start_temp);                 // Start position

                    End_temp[0] = cir_buffer_uart0[4] ;
                    End_temp[1] = cir_buffer_uart0[5] ;
                    End_postion=atoi(End_temp);                     //End position



                    if(Meter_Place.Slave_ID_flash>=Start_postion)
                    {
                        if(Meter_Place.Slave_ID_flash<= End_postion)
                        {

                            MAP_TimerDisable(TIMER6_BASE, TIMER_B);       //Disable fout counter
                            MAP_GPIOIntDisable(GPIO_PORTM_BASE, GPIO_INT_PIN_6);     //Enable the Individual Pin sensing intrrupt
                            MAP_IntDisable(INT_GPIOM);

                            Temp_pulses=0;
                            Flag.Pulse_Received = 0;
                            Edge_count=0;
                            pulse_count=0;          //New added after observation
                            Flag.Start_Pressed=0;
                            Error_Display_Seq=0;

                            if(reset_count_flag)
                            {
                                reset_count_flag=0;
                                disp_para_cnt=3;
                            }
                            reset_count=0;
                            if(Meter_Place.Slave_ID_flash== End_postion)
                            {
                                ENABLE_PIN();
                                send_acknowledgement();
                                DISABLE_PIN();
                            }
                         }

                    }

                    memset(Start_temp,'\0',sizeof(Start_temp));
                    memset(End_temp,'\0',sizeof(End_temp));
                    Clear_UART0_Buffer();

                    if(disp_para_cnt==4)
                    {
                        sRect_Label.i16XMin = 0;
                        sRect_Label.i16YMin = 130;                  // Clean Display
                        sRect_Label.i16XMax = 314;
                        sRect_Label.i16YMax = 180;
                        GrContextForegroundSet(&sContext_Label, BACKGROUND_COLOR);
                        GrRectFill(&sContext_Label, &sRect_Label);
                    }
                    if((disp_para_cnt==2)&&(result_display))
                    {
                        sRect_Label.i16XMin = 0;
                        sRect_Label.i16YMin = 130;                  // Clean Display
                        sRect_Label.i16XMax = 314;
                        sRect_Label.i16YMax = 180;
                        GrContextForegroundSet(&sContext_Label, BACKGROUND_COLOR);
                        GrRectFill(&sContext_Label, &sRect_Label);
                        result_display=0;
                    }



            break;


            case'O':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;
                    Start_postion=atoi(Start_temp);                 // Start position

                    End_temp[0] = cir_buffer_uart0[4] ;
                    End_temp[1] = cir_buffer_uart0[5] ;
                    End_postion=atoi(End_temp);                     //End position


                    if(Meter_Place.Slave_ID_flash== End_postion)
                    {
                        ENABLE_PIN();
                        send_acknowledgement();
                        DISABLE_PIN();
                    }
                    if(QD_Flag)
                    {
                        clean_flag=1;
                        QD_Flag=0;
                    }
                    transperent_mode=0;
                    memset(Start_temp,'\0',sizeof(Start_temp));
                    memset(End_temp,'\0',sizeof(End_temp));
                    echo_flag=0;
                    Clear_UART0_Buffer();

            break;

            case'P':

                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;
                    Start_postion=atoi(Start_temp);                 // Start position

                    End_temp[0] = cir_buffer_uart0[4] ;
                    End_temp[1] = cir_buffer_uart0[5] ;
                    End_postion=atoi(End_temp);                     //End position

                    if((Meter_Place.Slave_ID_flash >= Start_postion) && (Meter_Place.Slave_ID_flash <=End_postion))
                    {
                        DLMS_Temp_Index=25;length_query1=0;
                        while(cir_buffer_uart0[DLMS_Temp_Index]!='\r')
                        {
                            DLMS_Temp_buff[DLMS_Temp_Index-25] = cir_buffer_uart0[DLMS_Temp_Index] ;
                            DLMS_Temp_Index++;
                            length_query1++;
                        }
                    }
                    if(Meter_Place.Slave_ID_flash == End_postion)
                    {
                        ENABLE_PIN();
                        send_acknowledgement();
                        DISABLE_PIN();
                    }
                    BCC_check='3';
                    Clear_UART0_Buffer();           // Clear the receive buffer

            break;

            case'Q':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;
                    Start_postion=atoi(Start_temp);                 // Start position

                    End_temp[0] = cir_buffer_uart0[4] ;
                    End_temp[1] = cir_buffer_uart0[5] ;
                    End_postion=atoi(End_temp);                     //End position


                    if(Meter_Place.Slave_ID_flash== End_postion)
                    {
                        ENABLE_PIN();
                        send_acknowledgement();
                        DISABLE_PIN();
                    }

                    memset(Start_temp,'\0',sizeof(Start_temp));
                    memset(End_temp,'\0',sizeof(End_temp));
                    Clear_UART0_Buffer();

            break;

            case'R':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;
                    Start_postion=atoi(Start_temp);                 // Start position

                    End_temp[0] = cir_buffer_uart0[4] ;
                    End_temp[1] = cir_buffer_uart0[5] ;
                    End_postion=atoi(End_temp);                     //End position



                    if((Meter_Place.Slave_ID_flash >= Start_postion) && (Meter_Place.Slave_ID_flash <=End_postion))
                    {

                        // Processing the received command
                        ASCII_HEX_CONVERTER(length_query1);    // ASCII to hex conversion

                        for(i=0;i<hex_count;i++)
                        {
                          DLMS_Buffer[i]=C_DLMS_Buffer[i];
                        }

                        if(DLMS_Buffer[0]==0x7E && DLMS_Buffer[1]==0xA0)
                        {
                            length_query =(DLMS_Buffer[2]+2);
                            Command_length = length_query;
                            if((hex_count)==length_query)
                                complete_length=1;
                            else
                            {
                                complete_length=0;
                            }
                        }
                        else
                        {
                            complete_length=1;
                        }



                        // Send the Command to the Meter

                        Clear_UART1_Buffer();

                        if(Meter_Place.Slave_ID_flash== End_postion)    //Matched Position
                        {
                                if(complete_length)
                                {
                                    common_position_query();
                                }
                        }

                        if(Start_postion!=End_postion)
                        {
                            if((Meter_Place.Slave_ID_flash >=Start_postion)&& (Meter_Place.Slave_ID_flash<End_postion))
                            {
                                if(send_flag==0)
                                {
                                    if(complete_length)
                                    {
                                        common_position_query();
                                    }
                                }
                            }
                        }

                        send_flag=0;
                        Timeout_cnt=0;  // Timeout for HDLC screen display
                        memset(C_DLMS_Buffer,'\0',sizeof(C_DLMS_Buffer));
                        memset(Start_temp,'\0',sizeof(Start_temp));
                        memset(End_temp,'\0',sizeof(End_temp));

                        ok_length=0;

                    }

                        if(Meter_Place.Slave_ID_flash == End_postion)
                    {
                        ENABLE_PIN();
                        send_acknowledgement();
                        DISABLE_PIN();
                    }
                    Clear_UART0_Buffer();



            break;

            case'S':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;

                    Received_ID= atoi(Start_temp);


                    if(Meter_Data_Ready!=1)
                    {
                            response_meter();
                            retry_cnt++;
                            if(retry_cnt>50)
                            {
                               retry_cnt=0;
                               Meter_Data_Ready=1;
                            }
                    }




                    if((Meter_Place.Slave_ID_flash == Received_ID) && (Meter_Data_Ready ==1))
                    {

                        retry_cnt=0;
                        complete_data_flag=0;
                        HEX_ASCII_CONVERTER();
                        Clear_UART1_Buffer();

                        ENABLE_PIN();
                        send_DLMS_response();
                        DISABLE_PIN();

                        DLMS_Temp_Index=0;
                        length_query1=0;
                        Timeout_cnt=0;  // Timeout for HDLC screen display
                        Clear_UART0_Buffer();
                        Meter_Data_Ready =0;
                        response_timeout=0;

                     }
                    if((Meter_Place.Slave_ID_flash!=Received_ID))
                    {
                        Clear_UART0_Buffer();
                    }

                    memset(Start_temp,'\0',sizeof(Start_temp));


            break;

            case'T':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;
                    Start_postion=atoi(Start_temp);                 // Start position

                    End_temp[0] = cir_buffer_uart0[4] ;
                    End_temp[1] = cir_buffer_uart0[5] ;
                    End_postion=atoi(End_temp);                     //End position


                    if(Meter_Place.Slave_ID_flash== End_postion)
                    {
                        ENABLE_PIN();
                        send_acknowledgement();
                        DISABLE_PIN();
                    }

                    memset(Start_temp,'\0',sizeof(Start_temp));
                    memset(End_temp,'\0',sizeof(End_temp));
                    Clear_UART0_Buffer();

            break;


            case'Y':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;

                    Received_ID= atoi(Start_temp);
                    if( Meter_Place.Slave_ID_flash==Received_ID)
                    {
                        ENABLE_PIN();
                        send_status_ack();
                        DISABLE_PIN();

                    }
                    else
                    {
                        DISABLE_PIN();
                    }
                    memset(Start_temp,'\0',sizeof(Start_temp));
                    Clear_UART0_Buffer();

            break;






            case'z':
                    Start_temp[0] = cir_buffer_uart0[2] ;
                    Start_temp[1] = cir_buffer_uart0[3] ;
                    Start_postion=atoi(Start_temp);                 // Start position

                    End_temp[0] = cir_buffer_uart0[4] ;
                    End_temp[1] = cir_buffer_uart0[5] ;
                    End_postion=atoi(End_temp);

                    if((Meter_Place.Slave_ID_flash >= Start_postion) && (Meter_Place.Slave_ID_flash <=End_postion))
                    {
                        count=8;
                        while(cir_buffer_uart0[count]!='\r')
                        {
                            DLMS_Temp_buff[DLMS_Temp_Index-25] = cir_buffer_uart0[count] ;
                            count++;
                            DLMS_Temp_Index++;
                            length_query1++;
                        }
                    }

                    if(Meter_Place.Slave_ID_flash == End_postion)
                    {
                        ENABLE_PIN();
                        send_acknowledgement();
                        DISABLE_PIN();
                    }
                    BCC_check='2';
                    Clear_UART0_Buffer();           // Clear the receive buffer

            break;

            default:
                    Clear_UART0_Buffer();

            break;


        }

}

/******************************************************************************/
//      Function: Status measurement
//
/******************************************************************************/
void send_acknowledgement(void)
{
    pc_buff[0]=cir_buffer_uart0[0] ;//0x10;
    pc_buff[1]=cir_buffer_uart0[1] ;//0x01;
    pc_buff[2]='\r';
    for (i=0;i<3;i++)
    {
        UARTSend(&pc_buff[i],1);
        for (x=0;x<=20000;x++);
    }
}
/******************************************************************************/
//      Function: Hardware version
//
/******************************************************************************/
void send_version_HWACK(void)
{

    pc_buff[0]='D';
    pc_buff[1]='H';
    pc_buff[2]='V';
    pc_buff[3]='0';
    pc_buff[4]='2';
    pc_buff[5]='.';
    pc_buff[6]='5';
    pc_buff[7]='9';
    pc_buff[8]='\r';
    for (i=0;i<9;i++)
    {
        UARTSend(&pc_buff[i],1);
        for (x=0;x<=20000;x++);

    }

}

/******************************************************************************/
//      Function: Software version
//
/******************************************************************************/
void send_version_SWACK(void)
{
    pc_buff[0]='D';
    pc_buff[1]='I';
    pc_buff[2]='V';
    pc_buff[3]='0';
    pc_buff[4]='2';
    pc_buff[5]='.';
    pc_buff[6]='6';
    pc_buff[7]='5';
    pc_buff[8]='\r';
    for (i=0;i<9;i++)
    {
        UARTSend(&pc_buff[i],1);
        for (x=0;x<=20000;x++);
    }
}
/******************************************************************************/
//      Function: Slave ID Check
//
/******************************************************************************/

void Get_Slave_ID(void)
{
    memset(Temp_arrray,'\0', sizeof(Temp_arrray));
    Temp_arrray[0] = cir_buffer_uart0[2] ;
    Temp_arrray[1] = cir_buffer_uart0[3] ;
    Received_ID= atoi(Temp_arrray);

    memset(Temp_arrray,'\0', sizeof(Temp_arrray));

    Temp_arrray[0] = Meter_Place.Slave_ID[0] ;
    Temp_arrray[1] = Meter_Place.Slave_ID[1] ;
    Meter_Place.Slave_ID_flash= atoi(Temp_arrray);
}

/******************************************************************************/
//      Function: Send accuracy results
//
/******************************************************************************/
void send_accuracy_results(void)
{
    char sign;

        if(error_calc<0)
            sign='-';
        else
            sign='+';


        if(Higher_error_flag)
        {
            Higher_error_flag=0;
            C_error_buff[0]=0x18;
            C_error_buff[1]=0x18;
            C_error_buff[2]=0x18;
            C_error_buff[3]=0x18;
        }

        pc_buff[0]='D';
        pc_buff[1]='G';
        pc_buff[2]=sign;
        pc_buff[3]='?';
        pc_buff[4]=C_error_buff[0]+0x30;
        pc_buff[5]=C_error_buff[1]+0x30;
        pc_buff[6]=C_error_buff[2]+0x30;
        pc_buff[7]=C_error_buff[3]+0x30;
        pc_buff[8]='\r';
        for (i=0;i<9;i++)
        {
            UARTSend(&pc_buff[i],1);
            for (x=0;x<=20000;x++);
        }

}


/******************************************************************************/
//      Function: Send edge results
//
/******************************************************************************/
void send_edge_results(void)
{
    for(i=0;i<6;i++)
    {
        if(edge_buff[i]==0x20)
            edge_buff[i]=0x30;
    }

    pc_buff[0]='D';
    pc_buff[1]='G';
    pc_buff[2]=edge_buff[0];
    pc_buff[3]=edge_buff[1];
    pc_buff[4]=edge_buff[2];
    pc_buff[5]=edge_buff[3];
    pc_buff[6]=edge_buff[4];
    pc_buff[7]=edge_buff[5];
    pc_buff[8]='\r';
    for (i=0;i<9;i++)
    {
        UARTSend(&pc_buff[i],1);
        for (x=0;x<=20000;x++);
    }
}



/******************************************************************************/
//      Function: DLMS_response
//
/******************************************************************************/

void send_DLMS_response(void)
{
    unsigned int j;


        byte0=string_length%10;         //com
        string_length=string_length/10;    //com

        byte1=string_length%10;         //com
        string_length=string_length/10;    //com

        byte2=string_length%10;         //com
        string_length=string_length/10;    //com

        byte3=string_length%10;         //com
        string_length=string_length/10;    //com

        byte4=string_length%10;         //com
        string_length=string_length/10;     //com


        if(response_buffer[0]==0x7E)// && (response_buffer[1]==0xA0))
        {
            trans_length3= ((Receive_Query_length.data_value+2)*2);
        }
        else
        {
            trans_length3=(test_count_2*2);
        }

        pc_buff[0]=cir_buffer_uart0[0] ;
        pc_buff[1]=cir_buffer_uart0[1] ;
        pc_buff[2]=Start_temp[0] ;
        pc_buff[3]=Start_temp[1] ;
        pc_buff[4]='1';
        pc_buff[5]='S';
        pc_buff[6]='5';
        pc_buff[7]=BCC_check;
        pc_buff[8]=byte4+0x30;
        pc_buff[9]=byte3+0x30;
        pc_buff[10]=byte2+0x30;
        pc_buff[11]=byte1+0x30;
        pc_buff[12]=byte0+0x30;

    for (i=0;i<13;i++)
    {
        C_ascii_convert_buff[i]=pc_buff[i];
    }

    for (i=13,j=0;j<trans_length3;i++,j++)
    {
        C_ascii_convert_buff[i]=ascii_convert_buff[j];
    }


    for (i=0;i<(trans_length3+13);i++)
    {
        UARTSend(&C_ascii_convert_buff[i],1);
        for (x=0;x<=20000;x++);
    }

    pc_buff[0]='\r';
    UARTSend(&pc_buff[0],1);
    for (x=0;x<=20000;x++);
    Length_Received_Buff=0; test_count_2=0;
    Receive_Query_length.data_value=0;



}


/******************************************************************************/
//      Function: Stop Display
//
/******************************************************************************/
void Stop_display(void)
{
    Fix_Graphics();

    GrContextInit(&sContext_Meas_set[0][0], &g_sKentec320x240x16_SSD2119);
    sRect_Meas_set[0][0].i16XMin = 20;
    sRect_Meas_set[0][0].i16YMin = 176;
    sRect_Meas_set[0][0].i16XMax = 300;
    sRect_Meas_set[0][0].i16YMax = 204;

    GrContextBackgroundSet(&sContext_Meas_set[0][0], BACKGROUND_COLOR);   //Default display
    GrContextFontSet(&sContext_Meas_set[0][0], &g_sFontCmss28b);
    GrContextForegroundSet(&sContext_Meas_set[0][0], ClrGreen);
    GrStringDrawCentered(&sContext_Meas_set[0][0], "  Err: xxx.xx  %  ", -1,160,187, 1);       // Correct dispaly
}

/********************************************************************************************************/
//      void send_status_ack(void)
//
/********************************************************************************************************/
void send_status_ack(void)
{

    if(single_flag)
    {
       pc_buff[4]=measurement_status_flag+0x30; pc_buff[5]='0'; pc_buff[9]='0';
    }
    else
    {
       pc_buff[4]='0'; pc_buff[5]=measurement_status_flag+0x30; pc_buff[9]='0';
    }

    if(Edge_mode_flag)
    {
        pc_buff[4]='0'; pc_buff[5]='0'; pc_buff[9]='2';
    }

    if(reset_count_flag)
    {
        pc_buff[10]=reset_status_flag+0x30;
     }
     else
        pc_buff[10]='0';

    pc_buff[0]='D';
    pc_buff[1]='Y';
    pc_buff[2]=Meter_Place.Slave_ID[0];
    pc_buff[3]=Meter_Place.Slave_ID[1];
    pc_buff[6]='0';
    pc_buff[7]='0';
    pc_buff[8]='0';
    pc_buff[11]='0';
    pc_buff[12]='\r';

    for (i=0;i<13;i++)
    {
        UARTSend(&pc_buff[i],1);
        for (x=0;x<=20000;x++);
    }


}

/*******************************************************************************************************/
//void Response_To_PC(unsigned char* cir_buffer_uart0)
// Function send data to PC
/*******************************************************************************************************/
void Response_To_PC(unsigned char* cir_buffer_uart0)
{
    for (i=0;i<15;i++)
    {
        UARTSend_PC(&cir_buffer_uart0[i],1);
        for (x=0;x<=20000;x++);
    }

    memset(cir_buffer,'\0',sizeof(cir_buffer));
    write_ptr = cir_buffer;
    read_ptr = cir_buffer;
    rcv_buff_end = &cir_buffer[SIZE];

    memset(cir_buffer_uart0,'\0',sizeof(cir_buffer_uart0));
    write_ptr_uart0 = cir_buffer_uart0;
    read_ptr_uart0 = cir_buffer_uart0;
    rcv_buff_end_uart0 = &cir_buffer_uart0[SIZE];
 }

/*******************************************************************************************************/
/*******************************************************************************************************/
//void Constant_Pulse_Calc(void)
// Function fetch master meter consatnat and pulses from WINSAM
/*******************************************************************************************************/

void Constant_Pulse_Calc(void)
{
    char Temp_arrray1[7];

    for(count=8;count<14;count++)        //fetching Current
    {
        Temp_arrray1[count-8] = cir_buffer_uart0[count] ;
    }

    master_meter_constant = atof(Temp_arrray1);

    if(cir_buffer_uart0[count]=='1') master_meter_constant=master_meter_constant*10;
    else if(cir_buffer_uart0[count]=='2') master_meter_constant=master_meter_constant*100;
    else if(cir_buffer_uart0[count]=='3') master_meter_constant=master_meter_constant*1000;
    else if(cir_buffer_uart0[count]=='4') master_meter_constant=master_meter_constant*10000;
    else if(cir_buffer_uart0[count]=='5') master_meter_constant=master_meter_constant*100000;
    else if(cir_buffer_uart0[count]=='6') master_meter_constant=master_meter_constant*1000000;
    else if(cir_buffer_uart0[count]=='7') master_meter_constant=master_meter_constant*10000000;
    else if(cir_buffer_uart0[count]=='8') master_meter_constant=master_meter_constant*100000000;
    else if(cir_buffer_uart0[count]=='9') master_meter_constant=master_meter_constant*1000000000;

    N_Must= master_meter_constant;

    for(count=15;count<21;count++)        //fetching Current
    {
        Temp_arrray1[count-15] = cir_buffer_uart0[count];
    }

    master_meter_pulse =    atol(Temp_arrray1);


    if(cir_buffer_uart0[count]=='1') master_meter_pulse=master_meter_pulse*10;
    else if(cir_buffer_uart0[count]=='2') master_meter_pulse=master_meter_pulse*100;
    else if(cir_buffer_uart0[count]=='3') master_meter_pulse=master_meter_pulse*1000;
    else if(cir_buffer_uart0[count]=='4') master_meter_pulse=master_meter_pulse*10000;
    else if(cir_buffer_uart0[count]=='5') master_meter_pulse=master_meter_pulse*100000;
    else if(cir_buffer_uart0[count]=='6') master_meter_pulse=master_meter_pulse*1000000;
    else if(cir_buffer_uart0[count]=='7') master_meter_pulse=master_meter_pulse*10000000;
    else if(cir_buffer_uart0[count]=='8') master_meter_pulse=master_meter_pulse*100000000;
    else if(cir_buffer_uart0[count]=='9') master_meter_pulse=master_meter_pulse*1000000000;

    Temp_pulses=master_meter_pulse;





} 
/*******************************************************************************************************/

/*******************************************************************************************************/
//void ENABLE_PIN(void)
// Function enable the transmit Pin
/*******************************************************************************************************/
void ENABLE_PIN(void)
{
    MAP_GPIOPinWrite(GPIO_PORTD_BASE , GPIO_PIN_4 , GPIO_PIN_4);
//  for (x=0;x<=2000;x++);


}
/*******************************************************************************************************/
/*******************************************************************************************************/
//void DISABLE_PIN(void)
// Function enable the transmit Pin
/*******************************************************************************************************/
void DISABLE_PIN(void)
{
 
    MAP_GPIOPinWrite(GPIO_PORTD_BASE , GPIO_PIN_4 , 0);
//  for (x=0;x<=2000;x++);

}

/*******************************************************************************************************/ 

/*******************************************************************************************************/
//void Clear_UART0_Buffer(void)
// Function clear UART 0
/*******************************************************************************************************/
void Clear_UART0_Buffer(void)
{
    memset(cir_buffer_uart0,'\0', sizeof(cir_buffer_uart0));
    write_ptr_uart0 = &cir_buffer_uart0[0];
    read_ptr_uart0 = &cir_buffer_uart0[0];
    rcv_buff_end_uart0 = &cir_buffer_uart0[SIZE];
}

/*******************************************************************************************************/
//void Clear_UART1_Buffer(void)
// Function clear UART 0
/*******************************************************************************************************/
void Clear_UART1_Buffer(void)
{
    memset(cir_buffer,'\0',sizeof(cir_buffer));
    write_ptr = cir_buffer;
    read_ptr = cir_buffer;
    rcv_buff_end = &cir_buffer[SIZE];
}

/*******************************************************************************************************/
void single_continuous(void) 
{

    Start_temp[0] = cir_buffer_uart0[2] ;
    Start_temp[1] = cir_buffer_uart0[3] ;
    Start_postion=atoi(Start_temp);                 // Start position

    End_temp[0] = cir_buffer_uart0[4] ;
    End_temp[1] = cir_buffer_uart0[5] ;
    End_postion=atoi(End_temp);                     //End position

//  Eneterd_Slave_ID = atoi(Meter_Place.Slave_ID);
    if(Meter_Place.Slave_ID_flash>=Start_postion)
    {
        if(Meter_Place.Slave_ID_flash<= End_postion)

        {
            if(single_flag)
                Single_init();
            else if(continuous_flag)
                Continue_init();

            measurement_status_flag=1;
            pulse_received=1;
            Constant_Pulse_Calc();
            start_measurment();

            if(Meter_Place.Slave_ID_flash== End_postion)
            {
                ENABLE_PIN();
                send_acknowledgement();
                DISABLE_PIN();

            }


        }
    }


    if(cir_buffer_uart0[7]=='1') decimal_position=1;
    else if(cir_buffer_uart0[7]=='2') decimal_position=2;
    else if(cir_buffer_uart0[7]=='3') decimal_position=3;
    else if(cir_buffer_uart0[7]=='4') decimal_position=4;

    memset(Start_temp,'\0',sizeof(Start_temp));
    memset(End_temp,'\0',sizeof(End_temp));
    Clear_UART0_Buffer();

}


void Edge_count_fun(void)
{
    Start_temp[0] = cir_buffer_uart0[2] ;
    Start_temp[1] = cir_buffer_uart0[3] ;
    Start_postion=atoi(Start_temp);                 // Start position

    End_temp[0] = cir_buffer_uart0[4] ;
    End_temp[1] = cir_buffer_uart0[5] ;
    End_postion=atoi(End_temp);                     //End position

    Eneterd_Slave_ID = atoi(Meter_Place.Slave_ID);
    if(Meter_Place.Slave_ID_flash>=Start_postion)
    {
        if(Meter_Place.Slave_ID_flash<= End_postion)
        {

            start_edge_flag=1;
            Edge_count=0;
            C_Edge_count=0;
            pulse_count=0;          //New added after observation
            Edge_mode_flag=1;

            disp_para_cnt=3;

            if(Eneterd_Slave_ID== End_postion)
            {
                ENABLE_PIN();
                send_acknowledgement();
                DISABLE_PIN();

            }
         }

    }


    memset(Start_temp,'\0',sizeof(Start_temp));
    memset(End_temp,'\0',sizeof(End_temp));
    Clear_UART0_Buffer();

} 


void stop_error_measurment(void)
{
    for(count=2;count<4;count++)
    Start_temp[count-2] = cir_buffer_uart0[count] ;
    Start_postion=atoi(Start_temp);

    for(count=4;count<6;count++)
    End_temp[count-4] = cir_buffer_uart0[count] ;
    End_postion=atoi(End_temp);

    Eneterd_Slave_ID = atoi(Meter_Place.Slave_ID);
    if((Eneterd_Slave_ID>=Start_postion) && (Eneterd_Slave_ID<= End_postion))
    {
        if(Eneterd_Slave_ID== End_postion)
        {
            ENABLE_PIN();
            send_acknowledgement();
            DISABLE_PIN();
            Stop_display();
        }
        else
        {
            Stop_display();
        }
    }

    memset(Start_temp,'\0',sizeof(Start_temp));
    memset(End_temp,'\0',sizeof(End_temp));
    Clear_UART0_Buffer();
}


void ASCII_HEX_CONVERTER(unsigned char length_query5)
{
    unsigned char var_count,var_buff;
     hex_count=0;

        for(var_count=0,var_buff=0;var_count<length_query5;var_buff++)
        {
            if(DLMS_Temp_buff[var_count]>='0'&& DLMS_Temp_buff[var_count]<='9')
                DLMS_Temp_buff[var_count]=DLMS_Temp_buff[var_count]-0x30;
            else
                DLMS_Temp_buff[var_count]=DLMS_Temp_buff[var_count]-0x37;

            DLMS_Temp_buff[var_count]=  (DLMS_Temp_buff[var_count]<<4);
            var_count++;

            if(DLMS_Temp_buff[var_count]>='0'&& DLMS_Temp_buff[var_count]<='9')
                DLMS_Temp_buff[var_count]=DLMS_Temp_buff[var_count]-0x30;
            else
                DLMS_Temp_buff[var_count]=DLMS_Temp_buff[var_count]-0x37;

            C_DLMS_Buffer[var_buff]=(DLMS_Temp_buff[var_count-1]|DLMS_Temp_buff[var_count]);
            var_count++;
            hex_count++;
        }

}

unsigned char temp_var; 
void HEX_ASCII_CONVERTER(void)
{
    unsigned int var_count,var_buff,mode_length;
        if(trans_mode) mode_length= test_count_2;
        else
        mode_length=Receive_Query_length.data_value+2;
        mode_length=    string_length;
        test_count_2=string_length;
        for(var_count=0,var_buff=0;var_buff<mode_length;var_buff++)
        {
            temp_var=(response_buffer[var_buff]& 0xF0);
            temp_var=(temp_var>>4);

            if(temp_var<=9)
                ascii_convert_buff[var_count]=temp_var+0x30;
            else if(temp_var==0x0A)ascii_convert_buff[var_count]='A';
            else if(temp_var==0x0B)ascii_convert_buff[var_count]='B';
            else if(temp_var==0x0C)ascii_convert_buff[var_count]='C';
            else if(temp_var==0x0D)ascii_convert_buff[var_count]='D';
            else if(temp_var==0x0E)ascii_convert_buff[var_count]='E';
            else if(temp_var==0x0F)ascii_convert_buff[var_count]='F';

            var_count++;
            temp_var=   (response_buffer[var_buff]& 0x0F);

            if(temp_var<=9)
                ascii_convert_buff[var_count]=temp_var+0x30;
            else if(temp_var==0x0A)ascii_convert_buff[var_count]='A';
            else if(temp_var==0x0B)ascii_convert_buff[var_count]='B';
            else if(temp_var==0x0C)ascii_convert_buff[var_count]='C';
            else if(temp_var==0x0D)ascii_convert_buff[var_count]='D';
            else if(temp_var==0x0E)ascii_convert_buff[var_count]='E';
            else if(temp_var==0x0F)ascii_convert_buff[var_count]='F';

            var_count++;

        }
}



//**********************************************************************************
/// void DLMS_Query(void)
/// I/P: Nothing
/// O/P: Nothing
/// Remark: This function is first query of DLMS Protocol 
//**********************************************************************************
void DLMS_Query(void)
{
    unsigned int trans_length;

    if(DLMS_Buffer[0]==0x7E)// && DLMS_Buffer[1]==0xA0)
        trans_length=(DLMS_Buffer[2]+2);
    else
        trans_length= hex_count;


    for (i=0;i<trans_length;i++)
    {
        UARTSend_PC(&DLMS_Buffer[i],1);
        for (x=0;x<=20000;x++);
    }
}


void Single_init(void)
{
    Measuremnt_flag=1;          // Single Measuremnt
    single_flag=1;
    continuous_flag=0;
    start_edge_flag=0;
    Edge_mode_flag=0;
    DLMS_Read_Flag=0;


}

void Continue_init(void)
{
    Measuremnt_flag=0;          // Single Measuremnt
    single_flag=0;
    continuous_flag=1;
    start_edge_flag=0;
    Edge_mode_flag=0;
    DLMS_Read_Flag=0;



}

void response_meter(void)
{
    unsigned char Command_length_Temp;
    response_timeout=0;
//  while(Meter_Data_Ready == 0)
    {
        if(transperent_mode==0)
    //  if(cir_buffer[0]==0x7E)
        {
            for(i=0,ok_length=0; i< Command_length ; i++)
            {
                if(DLMS_Buffer[i]== cir_buffer[i])
                ok_length++;
            }

            if(ok_length == Command_length )
            {
                echo_flag=1;
                memset(response_buffer,'\0',sizeof(response_buffer));
                Command_length_Temp= Command_length;

                for(i=0; Command_length < Length_Received_Buff; i++,Command_length++)
                {
                    response_buffer[i]= cir_buffer[Command_length];
                    string_length++;
                }
                Command_length=Command_length_Temp;
                if(string_length >5)
                {
                    length_calculate();
                    if(string_length==(Receive_Query_length.data_value+2))
                //  if(string_length==(response_buffer[2]+2))
                    {
                        complete_data_flag=1;
                        Meter_Data_Ready =1;

                    }
                    else
                    {
                        string_length=0;
                        Meter_Data_Ready =0;
                    }
                }
            }
            if(ok_length != Command_length)
            {
                memset(response_buffer,'\0',sizeof(response_buffer));

                for(i=0;i < Length_Received_Buff; i++)
                {
                    response_buffer[i]= cir_buffer[i];
                    string_length++;
                }

                if(echo_flag==0)
                {
                    if(string_length >5)
                    {
                        length_calculate();
                        if(string_length==(Receive_Query_length.data_value+2))
                        //if(string_length ==(response_buffer[2]+2))
                        {
                            complete_data_flag=1;
                            Meter_Data_Ready =1;
                            response_timeout=0;
                        }
                        else
                        {
                            string_length=0;
                        }
                    }
                }
            }
        }

//  else
//  {
//              for(i=0,length_cnt=0;length_cnt<Length_Received_Buff ;i++,length_cnt++)
//                  {
//                      while(cir_buffer[length_cnt]!='\r')
//                      {
//                      }
//                          response_buffer[i]= cir_buffer[length_cnt];
//                          string_length++;
//                          length_cnt++;
//
//                  }
//
//                  trans_mode=1;
//                  test_count_2=i;
//                  Meter_Data_Ready =1;
//  }
//  if(response_timeout>5)
//  {
//          response_timeout=0;
//          Meter_Data_Ready =1;
//          break;
//
//  }
    }

}


void common_position_query(void)
{

    DLMS_Query();
    complete_length=0;
    if(DLMS_Buffer[0]==0x7E)// && DLMS_Buffer[1]==0xA0)
    {   send_flag=1;  DLMS_Read_Flag=1;
        if(once_display_hdlc==0)
        {
            once_display_hdlc=1;
            sRect_Label.i16XMin = 0;
            sRect_Label.i16YMin = 80;                   // previous value 0
            sRect_Label.i16XMax = 300;
            sRect_Label.i16YMax = 190;

            GrContextForegroundSet(&sContext_Label, BACKGROUND_COLOR);
            GrRectFill(&sContext_Label, &sRect_Label);
            GrContextBackgroundSet(&sContext_Label, BACKGROUND_COLOR);    //Default display
            GrContextFontSet(&sContext_Label, &g_sFontCmss44b);
            GrContextForegroundSet(&sContext_Label, ClrYellow);
        //  GrStringDraw(&sContext_Label," HDLC       ", -1,60,108 , 0);
            GrStringDraw(&sContext_Label," HDLC       ", -1,60,133 , 0);
        }
        GrContextBackgroundSet(&sContext_Label, BACKGROUND_COLOR);    //Default display
        GrContextFontSet(&sContext_Label, &g_sFontCmss40b);
        GrContextForegroundSet(&sContext_Label, BACKGROUND_COLOR);
        if(disp_cnt==0)
        GrStringDraw(&sContext_Label,"      ", -1,205,137 , 1);
        GrContextForegroundSet(&sContext_Label, ClrYellow);

        if(disp_cnt==1)
        GrStringDraw(&sContext_Label,".     ", -1,205,137 , 0); //153
        if(disp_cnt==2)
        GrStringDraw(&sContext_Label,". .   ", -1,205,137 , 0); //153
        if(disp_cnt==3)
        GrStringDraw(&sContext_Label,". . . ", -1,205,137 , 0);
        disp_cnt++;
        if(disp_cnt>3)disp_cnt=0;

   }


}

void display_string(void)
{
    if(cir_buffer_uart0[6]=='D')
    {
      if(cir_buffer_uart0[7]=='1')
        {
                sRect_Label.i16XMin = 0;
                sRect_Label.i16YMin = 130;                  // Clean Display
                sRect_Label.i16XMax = 314;
                sRect_Label.i16YMax = 180;
                GrContextForegroundSet(&sContext_Label, BACKGROUND_COLOR);
                GrRectFill(&sContext_Label, &sRect_Label);



                memset(lcd_disp,'\0',sizeof(lcd_disp));
            //  lcd_disp[0]=cir_buffer_uart0[11];
            //  lcd_disp[1]=cir_buffer_uart0[12];
                lcd_disp[0]=cir_buffer_uart0[13];
                lcd_disp[1]=cir_buffer_uart0[14];
                lcd_disp[2]=cir_buffer_uart0[15];
                lcd_disp[3]=cir_buffer_uart0[16];
                lcd_disp[4]='\0';

                GrContextFontSet(&sContext_Label, &g_sFontCmss44b);
                GrContextForegroundSet(&sContext_Label, ClrWhite);
                GrStringDrawCentered(&sContext_Label,lcd_disp,-1,GrContextDpyWidthGet(&sContext_Label) /2,153, 1);         //15 // previous value 23
                result_display=1;
                Error_Display_Seq=4;
            }
        }


}

void send_transperent_data(void)
{
    for(i=0;i<trans_data_cnt;i++)
    {
        UARTSend_PC(&cir_buffer_uart0[i] ,1);
        for (x=0;x<=20000;x++);
    }
    trans_data_cnt=0;
    Clear_UART0_Buffer();
}
void receive_transperent_data(void)
{
        ENABLE_PIN();
        for(i=0;i<Length_Received_Buff;i++)
        {
            UARTSend(&cir_buffer[i] ,1);
            for (x=0;x<=20000;x++);
        }
        DISABLE_PIN();
        Length_Received_Buff=0;
        Clear_UART1_Buffer();
}




 int Hex_To_Dec(unsigned char hexVal[]) 
{    
    int len =2;// strlen(hexVal); 
    int i;  
    // Initializing base value to 1, i.e 16^0 
    int base = 1; 
      
    int dec_val = 0; 
      
    // Extracting characters as digits from last character 
    for (i=len-1; i>=0; i--) 
    {    
        // if character lies in '0'-'9', converting  
        // it to integral 0-9 by subtracting 48 from 
        // ASCII value. 
        if (hexVal[i]>='0' && hexVal[i]<='9') 
        { 
            dec_val += (hexVal[i] - 48)*base; 
                  
            // incrementing base by power 
            base = base * 16; 
        } 
  
        // if character lies in 'A'-'F' , converting  
        // it to integral 10 - 15 by subtracting 55  
        // from ASCII value 
        else if (hexVal[i]>='A' && hexVal[i]<='F') 
        { 
            dec_val += (hexVal[i] - 55)*base; 
          
            // incrementing base by power 
            base = base*16; 
        } 
    } 
      
    return dec_val; 
} 
  

void length_calculate(void)
{
    Receive_Query_length.data_length[1]=response_buffer[1];
    Receive_Query_length.data_length[1]=Receive_Query_length.data_length[1]&0x07;
    Receive_Query_length.data_length[0]=response_buffer[2];

}



void tmipdisplay(void)
{
            sRect_Label.i16XMin = 0;
            sRect_Label.i16YMin = 80;                   // previous value 0
            sRect_Label.i16XMax = 300;
            sRect_Label.i16YMax = 190;
            GrContextForegroundSet(&sContext_Label, BACKGROUND_COLOR);
            GrRectFill(&sContext_Label, &sRect_Label);
            GrContextBackgroundSet(&sContext_Label, BACKGROUND_COLOR);    //Default display
            GrContextFontSet(&sContext_Label, &g_sFontCmss42b);
            GrContextForegroundSet(&sContext_Label, ClrLightCyan);

        //  GrStringDrawCentered(&sContext_Label,"- SCM -", -1,GrContextDpyWidthGet(&sContext_Label) / 2, 128, 0);
            GrStringDrawCentered(&sContext_Label,"- SCM -", -1,GrContextDpyWidthGet(&sContext_Label) / 2, 138, 0);
//          sprintf(pcBuffer,"- SCM - %d", Received_ID);
//          Null_char_position(6,&pcBuffer[0]);
//          GrStringDraw(&sContext_Label, pcBuffer, -1,80,108, 0);

}


void clear_display(void)
{
    sRect_Label.i16XMin = 0;
    sRect_Label.i16YMin = 80;                   // previous value 0
    sRect_Label.i16XMax = 300;
    sRect_Label.i16YMax = 190;
    GrContextForegroundSet(&sContext_Label, BACKGROUND_COLOR);
    GrRectFill(&sContext_Label, &sRect_Label);


    GrContextFontSet(&sContext_Label, &g_sFontCmss44b);
    GrContextForegroundSet(&sContext_Label, ClrRed);
    GrStringDrawCentered(&sContext_Label,"Error", -1,GrContextDpyWidthGet(&sContext_Label) / 2, 100, 0);
    GrContextForegroundSet(&sContext_Label, ClrWhite);
    GrStringDrawCentered(&sContext_Label,"   xxx.xx %  ", -1,GrContextDpyWidthGet(&sContext_Label) / 2,153 , 1);



}

